# -*- coding: utf-8 -*-
import os
import re
import warnings

import astropy.units as u
import numpy as np

from .custom_units import custom_units
from .log import get_log
from .logger import logger
from .quantity import NumpyQuantity
from .quantity import VPLANETQuantity as Quantity


class Output(object):
    """A class containing all of the information of a ``vplanet`` run."""

    def __init__(self):
        self.sysname = ""
        self.bodies = []

    def __getitem__(self, i):
        return self.bodies[i]

    def __len__(self):
        return len(self.bodies)

    def __repr__(self):
        return "<vplanet.Output: %s>" % self.sysname

    @property
    def members(self):
        """A list of all the properties of this object."""
        keys = list(self.__dict__.keys())
        return [key for key in keys if not key.startswith("_")]


class Body(object):
    """A class containing the parameter arrays of a body in a ``vplanet`` run.

    These are populated from either a ``.forward``, a ``.backward``, or a
    ``.Climate`` file.

    """

    def __init__(self):
        self._name = ""
        self._params = []

    def __getitem__(self, i):
        return self._params[i]

    def __len__(self):
        return len(self._params)

    def __repr__(self):
        return "<vplanet.Body: %s>" % self._name

    @property
    def members(self):
        """A list of all the properties of this object."""
        keys = list(self.__dict__.keys())
        return [key for key in keys if not key.startswith("_")]


def get_param_descriptions():
    """ """

    # Get the help message
    from .wrapper import help

    help_msg = str(help())

    # Get only the output params
    stroutput = help_msg.split("These options follow the argument saOutputOrder.")[1]
    stroutput = [x for x in stroutput.split("\n") if len(x)]
    description = {}
    for out in stroutput:
        if out.startswith("[-]"):
            try:
                n, d, _ = re.search(r"\[-\](.*) -- (.*). \[(.*)\]", out).groups()
            except AttributeError:
                raise ValueError(
                    "Error parsing the following line in the log file: {}".format(out)
                )
            description.update({n: d})
        else:
            try:
                n, d = re.search("(.*) -- (.*).", out).groups()
            except AttributeError:
                raise ValueError(
                    "Error parsing the following line in the log file: {}".format(out)
                )
            description.update({n: d})

    # Format the entries a bit
    for key, value in description.items():

        # Remove periods
        if value.endswith("."):
            description[key] = value[:-1]

        # Change "Time Rate of Change" to "Rate of Change"
        if value.lower().startswith("time rate of change"):
            description[key] = value[5:]

        # Remove "Body's"
        if value.lower().startswith("body's"):
            description[key] = value[7:]

    return description


def get_params(outputorder, file, units=True, body=None):
    """ """
    # Get parameter descriptions from the vplanet help
    description = get_param_descriptions()

    # Get params and units
    params_and_units = re.findall(r"(.*?)\[(.*?)\]", outputorder)

    # Populate the params
    params = []
    for j, param in enumerate(params_and_units):

        # Grab the array in the fwfile/bwfile
        array = []
        for line in file:
            array.append(float(line.split()[j]))

        # Get the name and units
        name = param[0].replace(" ", "")
        unit_str = param[1].replace(" ", "")

        # If the param name starts with a number,
        # add an underscore so we can make it a
        # valid class property name
        if any(name.startswith(str(n)) for n in range(10)):
            name = "_" + name

        # Process the unit
        if units:
            with warnings.catch_warnings(record=True) as w:
                try:
                    unit = u.Unit(unit_str)
                    assert len(w) == 0
                except ValueError as e:
                    logger.error(
                        "Error processing unit {}: ".format(unit_str) + (str(e))
                    )
                    unit = u.Unit("")
                except AssertionError:
                    logger.warn(
                        "Error processing unit {}: ".format(unit_str)
                        + str(w[0].message)
                    )
                    unit = u.Unit("")

            # Make it into an astropy quantity with units
            array = Quantity(np.array(array), unit=unit)
            physical_type = unit.physical_type

        else:

            # Keep it as a numpy array with tags. We'll
            # still keep track of the unit, but it's only
            # a passive tag!
            array = NumpyQuantity(np.array(array))
            array.unit = unit_str
            physical_type = None

        # Tag it for plotting
        array.tags = dict(
            name=name,
            description=description.get(name, name),
            body=body,
            physical_type=physical_type,
        )

        # Add to the list
        params.append(array)

    return params


def get_arrays(log, units=True):
    """ """
    # Initialize
    output = Output()

    # Get basic system info
    output.log = log
    output.sysname = log.sysname
    output.path = log.path

    # Grab body properties
    for i, body_name in enumerate(log._body_names):

        # Create a body
        body = Body()
        body._name = body_name
        body.name = body_name

        # Grab the input file name
        body.infile = getattr(log.header, "BodyFile%d" % (i + 1))

        # Grab the output file names
        body.fwfile = "%s.%s.forward" % (output.sysname, body._name)
        if not os.path.exists(os.path.join(output.path, body.fwfile)):
            body.fwfile = ""
        body.bwfile = "%s.%s.backward" % (output.sysname, body._name)
        if not os.path.exists(os.path.join(output.path, body.bwfile)):
            body.bwfile = ""
        body.climfile = "%s.%s.Climate" % (output.sysname, body._name)
        if not os.path.exists(os.path.join(output.path, body.climfile)):
            body.climfile = ""

        # Grab the forward arrays. Note that they may not exist for this body
        try:
            with open(os.path.join(output.path, body.fwfile), "r") as f:
                fwfile = f.readlines()
        except IOError:
            fwfile = [""]

        # Grab the backward arrays. Note that they may not exist for this body
        try:
            with open(os.path.join(output.path, body.bwfile), "r") as f:
                bwfile = f.readlines()
        except IOError:
            bwfile = [""]

        # TODO: Add support for *both* fwfile and bwfile at the same time?
        if fwfile != [""] and bwfile != [""]:
            logger.error(
                "Both a fwfile and a bwfile were detected. "
                + "Currently, vplanet can only handle one at a time. "
                + "Continuing, but ignoring the bwfile..."
            )

        # Now grab the output order and the params
        outputorder = getattr(log.initial, body._name).OutputOrder
        if fwfile != [""]:
            body._params = get_params(outputorder, fwfile, units=units, body=body._name)
        elif bwfile != [""]:
            body._params = get_params(outputorder, bwfile, units=units, body=body._name)

        # Climate file
        if body.climfile != "":
            # Grab the climate arrays...
            try:
                with open(os.path.join(output.path, body.climfile), "r") as f:
                    climfile = f.readlines()
            except IOError:
                raise Exception("Unable to open %s." % body.climfile)

            # ... and the grid order
            try:
                gridorder = getattr(log.initial, body._name).GridOutputOrder
                body._gridparams = get_params(
                    gridorder, climfile, units=units, body=body._name
                )
            except:
                logger.error(
                    "Unable to obtain grid output parameters from %s." % body.climfile
                )
                body._gridparams = []
        else:
            body._gridparams = []

        # Add the body
        output.bodies.append(body)

    return output


def get_output(path=".", sysname=None, units=True):
    """Parse all of the output from a :py:obj:`vplanet` run.

    Args:
        sysname (str, optional): System name. This is determined automatically,
            unless there are multiple runs in the same :py:obj:`path`. Defaults
            to None.
        path (str, optional): Path to the directory containing the results of
            the :py:obj:`vplanet` run. Defaults to the current directory.
        units (bool, optional): Whether or not the quantities returned by this
            method have astropy units. Default is True.

    Returns:
        A :py:class:`Output` instance containing all the information from the
        ``.log``, ``.forward``, and ``.backward`` output files.
    """
    # Get the log file and the arrays
    log = get_log(sysname=sysname, path=path, units=units)
    output = get_arrays(log, units=units)

    for body in output.bodies:

        # Make the body accessible as an attribute
        setattr(output, body._name, body)

        # Make all the arrays accessible as attributes
        for array in getattr(output, body._name)._params:
            setattr(getattr(output, body._name), array.tags["name"], array)

        # Grid params
        if len(getattr(output, body._name)._gridparams):

            # Get the time array
            iTime = np.argmax(
                [
                    array.tags["name"] == "Time"
                    for array in getattr(output, body._name)._gridparams
                ]
            )
            Time = getattr(output, body._name)._gridparams[iTime]

            # Get 2d array dimensions
            J = np.where(Time[1:] > Time[:-1])[0][0] + 1
            _, r = divmod(len(Time), J)
            if r != 0:
                raise Exception(
                    "Irregular time grid detected for {}. VPLANET is confused!".format(
                        body._name
                    )
                )

            for array in getattr(output, body._name)._gridparams:
                if array.tags["name"] != "Time":
                    # We don't want to overwrite the time array!
                    setattr(getattr(output, body._name), array.tags["name"], array)

    return output



# -*- coding: utf-8 -*-
import subprocess

_help = subprocess.check_output(["vplanet", "-h"]).decode("utf-8")
_help_verbose = subprocess.check_output(["vplanet", "-H"]).decode("utf-8")


class VPLANETHelp:
    """
    VPLANET help message wrapper.

    """

    def __init__(self, verbose=False):
        if verbose:
            self._help = _help_verbose
        else:
            self._help = _help

    def __repr__(self):
        return self._help

    def __str__(self):
        return self._help



# -*- coding: utf-8 -*-
import logging
import os
import re
import warnings
from glob import glob

import astropy.units as u
import numpy as np

from .custom_units import custom_units
from .logger import logger
from .quantity import VPLANETQuantity as Quantity


def get_param_unit(param, file, line):
    """
    Grab the parameter unit from a line in the log file.

    Returns an `astropy.units` unit.
    """
    groups = re.findall(r"\[(.*?)\]", param)
    if len(groups):
        unit_str = groups[-1]

        with warnings.catch_warnings(record=True) as w:
            unit = u.Unit("")
            try:
                unit = u.Unit(unit_str)
                assert len(w) == 0
            except ValueError:
                logger.error(
                    "Error processing line {} of {}: ".format(line, file)
                    + "Cannot interpret unit `{}`.".format(unit_str)
                )
            except AssertionError:
                logger.warn(
                    "Error processing line {} of {}: ".format(line, file)
                    + str(w[0].message)
                )

        return unit
    else:
        # Dimensionless
        return u.Unit("")


def get_param_name(param, file, line):
    """
    Grab the parameter name from a line in the log file.

    Returns a string.
    """
    # Replace any bad characters
    repl = [("#", "")]
    for a, b in repl:
        param = param.replace(a, b)

    # Search for a match
    match = re.search(r"\((.*?)\)", param)
    if match:
        param = match.groups()[0]
        # If the param name starts with a number,
        # add an underscore so we can make it a
        # valid class property name
        if any(param.startswith(str(n)) for n in range(10)):
            param = "_" + param
        return param
    else:
        # There is no (ParameterName) here.
        # This is therefore not a variable, but a setting.
        # If there are spaces, get rid of them and force title case
        if " " in param:
            param = param.title().replace(" ", "")
        return param


def get_param_value(val, unit, file, line, units=True):
    """
    Grab the parameter value from a line in the log file.

    Returns an int, float (with units), bool, None (if no value
    was provided) or a string (if processing failed).
    """
    # Remove spaces
    val = val.lstrip().rstrip()

    # Is there a value?
    if val == "":
        return None

    # Check if int, float, or bool
    int_chars = "-0123456789"
    float_chars = int_chars + ".+e"
    if all([c in int_chars for c in val]):
        try:
            val = int(val)
        except ValueError:
            logger.error(
                "Error processing line {} of {}: ".format(line, file)
                + "Cannot interpret value as integer."
            )
            # Return unprocessed string
            return val
    elif all([c in float_chars for c in val]):

        if units:

            try:
                val = Quantity(float(val) * unit)
            except ValueError:
                logger.error(
                    "Error processing line {} of {}: ".format(line, file)
                    + "Cannot interpret value as float."
                )
                # Return unprocessed string
                return val

        else:

            val = float(val)

    elif (val.lower() == "true") or (val.lower() == "yes"):
        val = True
    elif (val.lower() == "false") or (val.lower() == "no"):
        val = False
    elif val.lower() == "inf":
        val = np.inf
    elif val.lower() == "-inf":
        val = -np.inf
    elif val.lower() == "nan":
        val = np.nan
    return val


class Log(object):
    """
    A class containing the information of a `vplanet` `.log` output file.

    """

    def __init__(self):
        self.sysname = ""
        self._header = LogStage()
        self._header._name = "Header"
        self._initial = LogStage()
        self._initial._name = "Initial"
        self._final = LogStage()
        self._final._name = "Final"
        self._body_names = []
        self.path = ""

    @property
    def header(self):
        """
        Container for the information in the header of the log file,
        an instance of :py:class:`LogStage`.

        """
        return self._header

    @property
    def initial(self):
        """
        Container for the information about the initial conditions of the system,
        an instance of :py:class:`LogStage`.

        """

        return self._initial

    @property
    def final(self):
        """
        Container for the information about the final conditions of the system,
        an instance of :py:class:`LogStage`.

        """
        return self._final

    def __repr__(self):
        return "<vplanet.Log: %s>" % self.sysname

    @property
    def members(self):
        """A list of all the properties of this object."""
        keys = list(self.__dict__.keys())
        keys += ["header", "initial", "final"]
        return [key for key in keys if not key.startswith("_")]


class LogStage(object):
    """A class containing the information of a particular stage of
    a ``vplanet`` ``.log`` output file.

    """

    def __init__(self):
        self._name = ""

    def __repr__(self):
        return "<vplanet.LogStage: %s>" % self._name

    @property
    def members(self):
        """A list of all the properties of this object."""
        keys = list(self.__dict__.keys())
        return [key for key in keys if not key.startswith("_")]


class LogBody(object):
    """A class containing all of the properties of a body at a certain
    stage in a ``vplanet`` ``.log`` output file.

    """

    def __init__(self):
        self._name = ""

    def __repr__(self):
        return "<vplanet.LogBody: %s>" % self._name

    @property
    def members(self):
        """A list of all the properties of this object."""
        keys = list(self.__dict__.keys())
        return [key for key in keys if not key.startswith("_")]


def get_log(path=".", sysname=None, ext="log", units=True):
    """ """
    # Just in case!
    if ext.startswith("."):
        ext = ext[1:]

    # Look for the log file
    if sysname is None:
        lf = glob(os.path.abspath(os.path.join(path, "*.%s" % (ext))))
        if len(lf) > 1:
            raise Exception(
                "There are multiple log files matching {}.".format(
                    os.path.abspath(os.path.join(path, "*.%s" % (ext)))
                )
            )
        elif len(lf) == 0:
            raise Exception(
                "Could not find a log file matching {}.".format(
                    os.path.abspath(os.path.join(path, "*.%s" % (ext)))
                )
            )
        else:
            lf = lf[0]
            sysname = os.path.basename(lf).split(".%s" % ext)[0]
    else:
        lf = glob(os.path.join(path, "%s.%s" % (sysname, ext)))
        if len(lf) == 0:
            raise Exception(
                "Could not find a log file matching {}.".format(
                    os.path.abspath(os.path.join(path, "%s.%s" % (sysname, ext)))
                )
            )
        else:
            lf = lf[0]

    # Grab the contents
    with open(lf, "r") as f:
        lines = f.readlines()

    # Shorten the file name for logging
    lf = os.path.basename(lf)

    # Remove newlines and blank lines
    header = []
    initial = []
    final = []
    stage = 0
    for i, line in enumerate(lines):
        line = line.replace("\n", "")
        if "INITIAL SYSTEM PROPERTIES" in line:
            stage = 1
        elif "FINAL SYSTEM PROPERTIES" in line:
            stage = 2
        if len(line):
            if stage == 0:
                if ("Log file" not in line) and ("FORMATTING" not in line):
                    header.append((i + 1, line))
            elif stage == 1:
                if ("SYSTEM PROPERTIES" not in line) and ("PARAMETERS" not in line):
                    initial.append((i + 1, line))
            elif stage == 2:
                if ("SYSTEM PROPERTIES" not in line) and ("PARAMETERS" not in line):
                    final.append((i + 1, line))

    # Instantiate a `Log` object
    log = Log()
    log.sysname = sysname
    log.path = os.path.abspath(path)

    # Process the header
    for i, line in header:
        try:
            line_items = line.split(":")
            name_and_unit = line_items[0]
            value = ":".join(line_items[1:])
            unit = get_param_unit(name_and_unit, lf, i)
            name = get_param_name(name_and_unit, lf, i)
            value = get_param_value(value, unit, lf, i, units=units)
            setattr(log.header, name, value)
        except Exception as e:
            raise ValueError("Error processing line {} of {}: ".format(i, lf) + str(e))

    # Process the initial conditions
    body = "system"
    logbody = LogBody()
    logbody._name = body
    setattr(log.initial, body, logbody)
    for i, line in initial:
        if "BODY:" not in line:
            try:
                line_items = line.split(":")
                name_and_unit = line_items[0]
                value = ":".join(line_items[1:])
                unit = get_param_unit(name_and_unit, lf, i)
                name = get_param_name(name_and_unit, lf, i)
                value = get_param_value(value, unit, lf, i, units=units)
                setattr(getattr(log.initial, body), name, value)
            except Exception as e:
                raise ValueError(
                    "Error processing line {} of {}: ".format(i, lf) + str(e)
                )
        else:
            try:
                match = re.search(r"BODY:\s(.*?)\s-", line)
                body = match.groups()[0]
            except:
                raise ValueError(
                    "Error processing line {} of {}: ".format(i, lf)
                    + "Cannot understand body name."
                    + line
                )
            logbody = LogBody()
            logbody._name = body
            setattr(log.initial, body, logbody)
            log._body_names.append(body)

    # Process the final conditions
    body = "system"
    logbody = LogBody()
    logbody._name = body
    setattr(log.final, body, logbody)
    for i, line in final:
        if "BODY:" not in line:
            try:
                line_items = line.split(":")
                name_and_unit = line_items[0]
                value = ":".join(line_items[1:])
                unit = get_param_unit(name_and_unit, lf, i)
                name = get_param_name(name_and_unit, lf, i)
                value = get_param_value(value, unit, lf, i, units=units)
                setattr(getattr(log.final, body), name, value)
            except Exception as e:
                raise ValueError(
                    "Error processing line {} of {}: ".format(i, lf) + str(e)
                )
        else:
            try:
                match = re.search(r"BODY:\s(.*?)\s-", line)
                body = match.groups()[0]
            except:
                raise ValueError(
                    "Error processing line {} of {}: ".format(i, lf)
                    + "Cannot understand body name."
                    + line
                )
            logbody = LogBody()
            logbody._name = body
            setattr(log.final, body, logbody)

    return log



# -*- coding: utf-8 -*-
import logging

logger = logging.getLogger("vplanet")
ch = logging.StreamHandler()
ch.setLevel(logging.ERROR)
formatter = logging.Formatter("%(levelname)s:%(message)s")
ch.setFormatter(formatter)
logger.addHandler(ch)



# -*- coding: utf-8 -*-
import numbers
import re

import astropy
import astropy.units as u
import numpy as np
from astropy.units.core import Unit, UnitBase, UnitsError, dimensionless_unscaled
from astropy.utils.misc import isiterable

# TODO: There may be other methods in
# https://github.com/astropy/astropy/blob/master/astropy/units/quantity.py
# that we'll need to subclass in order to get `tags` to work


class VPLANETQuantity(u.Quantity):
    """A `~astropy.units.Quantity` represents a number with some associated unit.
    See also: http://docs.astropy.org/en/stable/units/quantity.html
    Parameters
    ----------
    value : number, `~numpy.ndarray`, `Quantity` object (sequence), str
        The numerical value of this quantity in the units given by unit.  If a
        `Quantity` or sequence of them (or any other valid object with a
        ``unit`` attribute), creates a new `Quantity` object, converting to
        `unit` units as needed.  If a string, it is converted to a number or
        `Quantity`, depending on whether a unit is present.
    unit : `~astropy.units.UnitBase` instance, str
        An object that represents the unit associated with the input value.
        Must be an `~astropy.units.UnitBase` object or a string parseable by
        the :mod:`~astropy.units` package.
    dtype : ~numpy.dtype, optional
        The dtype of the resulting Numpy array or scalar that will
        hold the value.  If not provided, it is determined from the input,
        except that any integer and (non-Quantity) object inputs are converted
        to float by default.
    copy : bool, optional
        If `True` (default), then the value is copied.  Otherwise, a copy will
        only be made if ``__array__`` returns a copy, if value is a nested
        sequence, or if a copy is needed to satisfy an explicitly given
        ``dtype``.  (The `False` option is intended mostly for internal use,
        to speed up initialization where a copy is known to have been made.
        Use with care.)
    order : {'C', 'F', 'A'}, optional
        Specify the order of the array.  As in `~numpy.array`.  This parameter
        is ignored if the input is a `Quantity` and ``copy=False``.
    subok : bool, optional
        If `False` (default), the returned array will be forced to be a
        `Quantity`.  Otherwise, `Quantity` subclasses will be passed through,
        or a subclass appropriate for the unit will be used (such as
        `~astropy.units.Dex` for ``u.dex(u.AA)``).
    ndmin : int, optional
        Specifies the minimum number of dimensions that the resulting array
        should have.  Ones will be pre-pended to the shape as needed to meet
        this requirement.  This parameter is ignored if the input is a
        `Quantity` and ``copy=False``.
    Raises
    ------
    TypeError
        If the value provided is not a Python numeric type.
    TypeError
        If the unit provided is not either a :class:`~astropy.units.Unit`
        object or a parseable string unit.
    Notes
    -----
    Quantities can also be created by multiplying a number or array with a
    :class:`~astropy.units.Unit`. See http://docs.astropy.org/en/latest/units/
    """

    # Need to set a class-level default for _equivalencies, or
    # Constants can not initialize properly
    _equivalencies = []

    # Default unit for initialization; can be overridden by subclasses,
    # possibly to `None` to indicate there is no default unit.
    _default_unit = dimensionless_unscaled

    # Ensures views have an undefined unit.
    _unit = None

    __array_priority__ = 10000

    def __new__(
        cls,
        value,
        unit=None,
        tags={},
        dtype=None,
        copy=True,
        order=None,
        subok=False,
        ndmin=0,
    ):

        if unit is not None:
            # convert unit first, to avoid multiple string->unit conversions
            unit = Unit(unit)

        # optimize speed for Quantity with no dtype given, copy=False
        if isinstance(value, u.Quantity):
            if unit is not None and unit is not value.unit:
                value = value.to(unit)
                # the above already makes a copy (with float dtype)
                copy = False

            if type(value) is not cls and not (subok and isinstance(value, cls)):
                value = value.view(cls)

            if dtype is None:
                if not copy:
                    return value

                if value.dtype.kind in "iu":
                    dtype = float

            return np.array(
                value,
                dtype=dtype,
                copy=copy,
                order=order,
                subok=True,
                ndmin=ndmin,
            )

        # Maybe str, or list/tuple of Quantity? If so, this may set value_unit.
        # To ensure array remains fast, we short-circuit it.
        value_unit = None
        if not isinstance(value, np.ndarray):
            if isinstance(value, str):
                # The first part of the regex string matches any integer/float;
                # the second parts adds possible trailing .+-, which will break
                # the float function below and ensure things like 1.2.3deg
                # will not work.
                pattern = (
                    r"\s*[+-]?"
                    r"((\d+\.?\d*)|(\.\d+)|([nN][aA][nN])|"
                    r"([iI][nN][fF]([iI][nN][iI][tT][yY]){0,1}))"
                    r"([eE][+-]?\d+)?"
                    r"[.+-]?"
                )

                v = re.match(pattern, value)
                unit_string = None
                try:
                    value = float(v.group())

                except Exception:
                    raise TypeError(
                        'Cannot parse "{}" as a {}. It does not '
                        "start with a number.".format(value, cls.__name__)
                    )

                unit_string = v.string[v.end() :].strip()
                if unit_string:
                    value_unit = Unit(unit_string)
                    if unit is None:
                        unit = value_unit  # signal no conversion needed below.

            elif (
                isiterable(value)
                and len(value) > 0
                and all(isinstance(v, VPLANETQuantity) for v in value)
            ):
                # Convert all quantities to the same unit.
                if unit is None:
                    unit = value[0].unit
                value = [q.to_value(unit) for q in value]
                value_unit = unit  # signal below that conversion has been done

        if value_unit is None:
            # If the value has a `unit` attribute and if not None
            # (for Columns with uninitialized unit), treat it like a quantity.
            try:
                value_unit = value.unit
            except AttributeError:
                value_unit = None
            if value_unit is None:
                # Default to dimensionless for no (initialized) unit attribute.
                if unit is None:
                    unit = cls._default_unit
                value_unit = unit  # signal below that no conversion is needed
            else:
                try:
                    value_unit = Unit(value_unit)
                except Exception as exc:
                    raise TypeError(
                        "The unit attribute {!r} of the input could "
                        "not be parsed as an astropy Unit, raising "
                        "the following exception:\n{}".format(value.unit, exc)
                    )

                if unit is None:
                    unit = value_unit
                elif unit is not value_unit:
                    copy = False  # copy will be made in conversion at end

        value = np.array(
            value,
            dtype=dtype,
            copy=copy,
            order=order,
            subok=False,
            ndmin=ndmin,
        )

        # check that array contains numbers or long int objects
        if value.dtype.kind in "OSU" and not (
            value.dtype.kind == "O" and isinstance(value.item(0), numbers.Number)
        ):
            raise TypeError("The value must be a valid Python or Numpy numeric type.")

        # by default, cast any integer, boolean, etc., to float
        if dtype is None and value.dtype.kind in "iuO":
            value = value.astype(float)

        # if we allow subclasses, allow a class from the unit.
        if subok:
            try:
                qcls = unit._quantity_class
            except AttributeError:
                qcls = cls
            if issubclass(qcls, cls):
                cls = qcls

        value = value.view(cls)
        value._set_unit(value_unit)

        # Custom tags
        value.tags = tags

        if unit is value_unit:
            return value
        else:
            # here we had non-Quantity input that had a "unit" attribute
            # with a unit different from the desired one.  So, convert.
            return value.to(unit)

    def __array_finalize__(self, obj):
        # If we're a new object or viewing an ndarray, nothing has to be done.
        if obj is None or obj.__class__ is np.ndarray:
            return

        # If our unit is not set and obj has a valid one, use it.
        if self._unit is None:
            try:
                unit = obj._unit
            except AttributeError:
                unit = None
            else:
                self._set_unit(unit)

        # Copy info if the original had `info` defined.  Because of the way the
        # DataInfo works, `'info' in obj.__dict__` is False until the
        # `info` attribute is accessed or set.
        if "info" in obj.__dict__:
            self.info = obj.info

        # Custom tags
        try:
            self.tags = obj.tags
        except AttributeError:
            self.tags = {}


class NumpyQuantity(np.ndarray):
    """
    A custom subclass of numpy ndarray with tags.

    """

    def __new__(cls, input_array, tags={}, unit=None):
        # Input array is an already formed ndarray instance
        # We first cast to be our class type
        obj = np.asarray(input_array).view(cls)
        # add the new attribute to the created instance
        obj.tags = tags
        obj.unit = unit
        # Finally, we must return the newly created object:
        return obj

    def __array_finalize__(self, obj):
        # see InfoArray.__array_finalize__ for comments
        if obj is None:
            return
        try:
            self.tag = obj.tags
        except AttributeError:
            self.tag = {}
        try:
            self.unit = obj.unit
        except AttributeError:
            self.unit = None

    def __array_wrap__(self, out_arr, context=None):
        # Call the parent
        return np.ndarray.__array_wrap__(self, out_arr, context)



import os
import sys
from distutils.command.clean import clean
from glob import glob

from setuptools import Extension, find_packages, setup
from setuptools.command.build_ext import build_ext
from setuptools.command.develop import develop

# Read current code version
VersionPath = os.path.join(os.path.abspath(os.path.dirname(__file__)), "VERSION")
print(VersionPath)
VERSION = open(VersionPath, "r").read().split("\n")[0].strip()


class BuildExt(build_ext):
    """A custom build extension for adding compiler-specific options."""

    def build_extensions(self):
        ct = self.compiler.compiler_type  # msvc or unix
        for ext in self.extensions:
            if ct == "msvc":
                ext.extra_compile_args = ["/EHsc"]
            else:
                ext.extra_compile_args = [
                    "-Wno-unused-variable",
                    "-Wno-missing-braces",
                    "-Wno-strict-prototypes",
                    "-Wno-sign-compare",
                    "-Wno-comment",
                ]
        build_ext.build_extensions(self)


class Develop(develop):
    """Custom develop command that clears build cache before install."""

    def run(self):
        c = clean(self.distribution)
        c.all = True
        c.finalize_options()
        c.run()
        develop.run(self)


macros = [
    ("VPLANET_PYTHON_INTERFACE", 1),
    (
        "VPLANET_VERSION",
        '"{}"'.format(VERSION),
    ),
]
if sys.platform.startswith("win"):
    macros += [("VPLANET_ON_WINDOWS", 1)]

ext_modules = [
    Extension(
        "vplanet.vplanet_core",
        glob("src/*.c"),
        include_dirs=["src"],
        language="c",
        define_macros=macros,
    )
]
cmdclass = {"build_ext": BuildExt, "develop": Develop}

# Vplanet suite of tools
vplanet_suite = [
    "vplot>=1.0.5",
    "vspace>=2.3.2",
    "bigplanet>=3.0.1",
    "multiplanet>=2.0.3",
]

setup(
    name="vplanet",
    author="Rory Barnes",
    author_email="rkb9@uw.edu",
    url="https://github.com/VirtualPlanetaryLaboratory/vplanet",
    version=VERSION,
    description="The virtual planet simulator",
    long_description=open("README.md", "r").read(),
    long_description_content_type="text/markdown",
    license="MIT",
    packages=["vplanet"],
    # install_requires=vplanet_suite + ["astropy>=3.0", "numpy", "tqdm", "pre-commit"],
    install_requires=vplanet_suite + ["astropy>=3.0", "numpy", "tqdm", "seaborn"],
    python_requires=">=3.6",
    # use_scm_version={
    # "write_to": os.path.join("vplanet", "vplanet_version.py"),
    # "write_to_template": '__version__ = "{version}"\n',
    # },
    ext_modules=ext_modules,
    cmdclass=cmdclass,
    include_package_data=True,
    zip_safe=False,
    entry_points={"console_scripts": ["vplanet=vplanet.wrapper:_entry_point"]},
)

# sub.call(["pre-commit", "install"])
